import React, { Component } from "react";
import { connect } from "react-redux";
import { Select, MenuItem, IconButton } from "@material-ui/core";
import {
  NavigateNext as NavigateNextIcon,
  NavigateBefore as NavigateBeforeIcon
} from "@material-ui/icons";
import {
  paginationPropType,
  widthNumberPropType,
  rowsPerPagePropType,
  setPagePagePropType,
  setRowsPerPagePropType
} from "../../proptypes";
import {
  setPage as setPageAction,
  setRowsPerPage as setRowsPerPageAction
} from "../../redux/actions/datatableActions";

class DatatableFooter extends Component {

  handlePageChange = (nextPage) => {
    console.log("Updating page on click...")
    const pageAttributes = {
      nextPage,
      pagination: this.props.pagination
    }
    this.props.onPageChange(pageAttributes);
    this.props.setPage(nextPage)
  }

  componentDidMount() {
    console.log("Mounting...", this.props.startPage)
    this.props.setPage(this.props.startPage)
  }

  render() {
    const {
      width,
      rowsPerPage,
      pagination,
      setPage,
      setRowsPerPage,
      onPageChange,
      getNumPages
    } = this.props;

    console.log("Updating: ", pagination, this.props.startPage)
    return (
      <div className="Footer" style={{ width }}>
        <div className="Footer-Element">
          Rows :
          <Select
            className="select-rowsPerPage"
            value={pagination.rowsPerPageSelected}
            onChange={e => setRowsPerPage(e.target.value)}
            disabled={
              rowsPerPage.available.length === 1 || pagination.pageTotal === 0
            }
          >
            {rowsPerPage.available.map(val => (
              <MenuItem key={val} value={val}>
                {val}
              </MenuItem>
            ))}
          </Select>
        </div>

        <div className="Footer-Element">
          Page :
          <Select
            className="select-page"
            value={pagination.pageSelected}
            onChange={e => this.handlePageChange(e.target.value)}
            disabled={pagination.pageTotal === 1 || pagination.pageTotal === 0}
          >
            {Array.apply(null, { length: pagination.pageTotal })
              .map(Number.call, Number)
              .map(val => (
                <MenuItem key={val} value={val + 1}>
                  {val + 1}
                </MenuItem>
              ))}
          </Select>
        </div>

        <div className="Footer-Element">
          <IconButton
            className="previous-page"
            disabled={
              pagination.pageSelected === 1 || pagination.pageTotal === 0
            }
            onClick={() => this.handlePageChange(pagination.pageSelected - 1)}
          >
            <NavigateBeforeIcon />
          </IconButton>
          <IconButton
            className="next-page"
            disabled={
              pagination.pageSelected === pagination.pageTotal ||
              pagination.pageTotal === 0
            }
            onClick={() => this.handlePageChange(pagination.pageSelected + 1)}
          >
            <NavigateNextIcon />
          </IconButton>
        </div>
      </div>
    );
  }
}

DatatableFooter.propTypes = {
  pagination: paginationPropType.isRequired,
  width: widthNumberPropType.isRequired,
  rowsPerPage: rowsPerPagePropType.isRequired,
  setPage: setPagePagePropType,
  setRowsPerPage: setRowsPerPagePropType
};

const mapDispatchToProps = dispatch => {
  return {
    setPage: pageNumber => dispatch(setPageAction(pageNumber)),
    setRowsPerPage: rowsPerPage => dispatch(setRowsPerPageAction(rowsPerPage))
  };
};

const mapStateToProps = state => {
  return {
    width: state.datatableReducer.dimensions.datatable.widthNumber,
    pagination: state.datatableReducer.pagination,
    rowsPerPage: state.datatableReducer.features.rowsPerPage
  };
};

export default connect(
  mapStateToProps,
  mapDispatchToProps
)(DatatableFooter);
